express = require 'express'
glob = require 'glob'

favicon = require 'serve-favicon'
logger = require 'morgan'
cookieParser = require 'cookie-parser'
bodyParser = require 'body-parser'
compress = require 'compression'
methodOverride = require 'method-override'

module.exports = (app, config) ->
  env = process.env.NODE_ENV || 'development'
  app.locals.ENV = env
  app.locals.ENV_DEVELOPMENT = env == 'development'
  

  # app.use(favicon(config.root + '/public/img/favicon.ico'))
  app.use logger 'dev'
  app.use bodyParser.json()
  app.use bodyParser.urlencoded(
    extended: true
  )
  app.use cookieParser()
  app.use compress()
  app.use express.static config.root + '/public'
  app.use methodOverride()

  controllers = glob.sync config.root + '/app/controllers/**/*.coffee'
  controllers.forEach (controller) ->
    require(controller)(app)

  # catch 404 and forward to error handler
  app.use (req, res, next) ->
    err = new Error 'Not Found'
    err.status = 404
    next err

  # error handlers

  # development error handler
  # will print stacktrace
  
  errorTemplate = require('marko').load require.resolve '../app/views/error.marko'
  if app.get('env') == 'development'
    app.use (err, req, res) ->
      res.status err.status || 500
      errorTemplate.render
        message: err.message,
        error: err
        title: 'error'
      , res

  # production error handler
  # no stacktraces leaked to user
  app.use (err, req, res) ->
    res.status err.status || 500
    errorTemplate.render
      message: err.message,
      error: {}
      title: 'error'
    , res
